#!/usr/bin/python
#
# GUI for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>

import sys, os
import time

saveargs = []
for arg in sys.argv[1:]:
    saveargs.append(arg)
    sys.argv.remove(arg)

import gtk, GTK, GDK
from gtk import TRUE
from gtk import FALSE
from gtk import gdk_beep
import gnome, gnome.ui
import libglade
import signal
import rpm
import lilocfg

sys.path.append("/usr/share/rhn/up2date")

from translate import _, N_, cat

import progress
import advisorywin
import checklist
import up2date

class Gui:

    def __init__(self):
        self.gotPackages = FALSE
        self.gotSkippedPackages = FALSE
        
        self.xml = libglade.GladeXML("/usr/share/rhn/up2date/gui.glade", "mainWin", domain="up2date")
        self.xml.signal_autoconnect (
            { "onDruidCancel" : self.onDruidCancel,
              "onStartPageNext" : self.onStartPageNext,
              "onSkippedPagePrepare" : self.onSkippedPagePrepare,
              "onSkippedPageNext" : self.onSkippedPageNext,
              "onPackagePagePrepare" : self.onPackagePagePrepare,
              "onPackagePageNext" : self.onPackagePageNext,
              "onPackagePageBack" : self.onPackagePageBack,
              "onDependencyPagePrepare" : self.onDependencyPagePrepare,
              "onRetrievalPagePrepare" : self.onRetrievalPagePrepare,
              "onRetrievalPageNext" : self.onRetrievalPageNext,
              "onRetrievalPageBack" : self.onRetrievalPageBack,
              "onInstallPagePrepare": self.onInstallPagePrepare,
              "onInstallPageBack": self.onInstallPageBack,
              "onFinishPagePrepare" : self.onFinishPagePrepare,
              "onFinishPageFinish" : self.onFinishPageFinish } )

        self.druid = self.xml.get_widget("druid")
        self.mainWin = self.xml.get_widget("mainWin")
        self.mainWin.connect("delete-event", gtk.mainquit)
        self.mainWin.connect("hide", gtk.mainquit)
        self.setupSkippedList()
        self.setupPackageList()

        # some bold labels
        label = self.xml.get_widget("pkgInfoLabel1")
        style = label.get_style().copy()
        style.font = gtk.load_font("-*-helvetica-bold-r-normal--12-*-*-*-p-*-iso8859-1")
        label.set_style(style)
        label = self.xml.get_widget("pkgInfoLabel2")
        label.set_style(style)

        # background on finished page
        box = self.xml.get_widget("finishEventBox")
        style = box.get_style().copy()
        color = gtk.GdkColor(52232, 0, 0)
        style.bg[GTK.STATE_NORMAL] = color
        box.set_style(style)
        
        self.mainWin.show_all()
        self.xml.get_widget("packagesFinishedLabel").set_text("...")
        self.xml.get_widget("installingFinishedLabel").set_text("...")
        

    def onDruidCancel(self, dummy):
        gtk.mainquit()


    def fatalError(self, error):
        dlg = gnome.ui.GnomeErrorDialog(error, self.mainWin)
        dlg.run_and_close()
        
        gtk.mainquit()
        sys.exit(1)


    def setBusyCursor(self):
        cursor = gtk.cursor_new (GDK.WATCH)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def setArrowCursor(self):
        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)
            

    def updateSelectedSize(self, state, rowData):
        sizeLabel = self.xml.get_widget("sizeLabel")
         
        if state == 0:
            self.totalSelectedSize = self.totalSelectedSize - rowData[4]
        else:
            self.totalSelectedSize = self.totalSelectedSize + rowData[4]
        sizeLabel.set_text("%d kb" % (self.totalSelectedSize / 1024))
        if self.totalSelectedSize > 0 and not self.pkgsSelected:
            self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
            self.pkgsSelected = TRUE
        elif self.totalSelectedSize == 0 and self.pkgsSelected:
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            self.pkgsSelected = FALSE
            
    def setupSkippedList(self):
        self.currentRow = -1
        window = self.xml.get_widget("skippedWindow")
        
        self.skippedArea = checklist.CheckList(5)
        self.skippedArea.set_column_title(1, _("Package Name"))
        self.skippedArea.set_column_min_width(1, 200)
        self.skippedArea.set_column_title(2, _("Version"))
        self.skippedArea.set_column_min_width(2, 50)
        self.skippedArea.set_column_title(3, _("Rel."))
        self.skippedArea.set_column_min_width(3, 50)
        self.skippedArea.set_column_title(4, _("Size"))
        self.skippedArea.set_column_title(5, _("Reason Skipped"))
        self.skippedArea.set_column_min_width(4, 50)
        self.skippedArea.column_titles_show()
        self.skippedArea.focus_row = -1

        self.skippedArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("skippedPackageInfoLabel")))
        window.add(self.skippedArea)

        self.xml.get_widget("skippedSelectAllButton").connect("toggled",
                                                              self.selectAll)
        self.xml.get_widget("advisoryButton").connect("clicked",
                                                      self.showAdvisory,
                                                      self.skippedArea)

        
    def setupPackageList(self):
        self.currentRow = -1
        window = self.xml.get_widget("packageWindow")
        
        self.packageArea = checklist.CheckList(4)
        self.packageArea.set_column_title(1, _("Package Name"))
        self.packageArea.set_column_min_width(1, 200)
        self.packageArea.set_column_title(2, _("Version"))
        self.packageArea.set_column_min_width(2, 50)
        self.packageArea.set_column_title(3, _("Release"))
        self.packageArea.set_column_min_width(3, 50)
        self.packageArea.set_column_title(4, _("Size"))
        self.packageArea.column_titles_show()
        self.packageArea.set_toggled_func(self.updateSelectedSize)
        self.packageArea.focus_row = -1

        self.packageArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("packageInfoLabel")))
        window.add(self.packageArea)

        self.xml.get_widget("selectAllButton").connect("toggled",
                                                       self.selectAll)

        self.xml.get_widget("advisoryButton2").connect("clicked",
                                                       self.showAdvisory,
                                                       self.packageArea)

                                                    

    def showAdvisory(self, button, area):
        if self.currentRow == -1:
            gdk_beep()
            return

        self.setBusyCursor()
        #awin = advisorywin.AdvisoryWin()

        rowData = area.get_row_data(self.currentRow)
        advisories = up2date.getAdvisoryInfo(rowData[1])
        self.setArrowCursor()
        if not advisories:
            dlg = gnome.ui.GnomeOkDialog(_("No advisory information available."), self.mainWin)
            dlg.run_and_close()
            return

        for a in advisories:
            txt = "%s [%s]\n%s\n--------\n%s" % (a['errata_type'],
                                   a['advisory'],
                                   a['topic'],
                                   a['description'])
            dlg = gnome.ui.GnomeOkDialog(txt, self.mainWin)
            dlg.run_and_close()
            
            

    def selectAll(self, button):
        if button.get_name() == "selectAllButton":
            area = self.packageArea
        elif button.get_name() == "skippedSelectAllButton":
            area = self.skippedArea
        else:
            return
        for index in range(area.n_rows):
            rowData = area.get_row_data(index)
            if rowData[0] != button.get_active():
                area._toggle_row(index)


    def showPackageInfo(self, area, row, column, event, label):
        self.currentRow = row
        rowData = area.get_row_data(row)
        hdr = up2date.getHeader(rowData[1],lite=1)
        label.set_text(hdr['summary'] + "\n" + hdr['description'])

    def __refreshCallback(self):
        while gtk.events_pending():
                gtk.mainiteration(FALSE)

    def onStartPageNext(self, page, dummy):
        if not self.gotPackages:
            self.pkgList = []
            self.selectedPkgList = []
            self.skipPkgList = []
            self.pkgsSelected = FALSE
            self.totalSelectedSize = long(0)
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            
            pwin = progress.Progress()
            pwin.setLabel(_("Red Hat Update Agent is building a list of updated\nRPM packages installed on your system.  Please wait."))

            try:
                up2date.maybeUpdateVersion()
                # override whatever this value is so we get good
                # interactive GUI response
                up2date.cfg.writeEntry("headerFetchCount", 20)
                (self.pkgList, self.skipPkgList) = up2date.getUpdatedPackageList([],pwin.setLabel, pwin.setProgress, self.__refreshCallback)
            except up2date.RpmError, e:
                pwin.destroy()
                self.fatalError(_("There was a fatal RPM error.  The message was:\n") + e.errmsg)
            except up2date.CommunicationError, e:
                pwin.destroy()
                self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
            except OSError, e:
                self.fatalError(_("An unexpected OS error occurred:\n%s") % e)

            pwin.destroy()
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

            self.gotPackages = TRUE

        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("packagePage"))
            
        return TRUE

    def onSkippedPagePrepare(self, page, dummy):
        if not self.gotSkippedPackages:
            self.skippedArea.freeze()
            for pkg,reason in self.skipPkgList:
                try:
                    self.skippedArea.append_row((pkg[0], pkg[1],
                                                 pkg[2],
                                                 "%d kb" % (pkg[4] / 1024),
                                                 reason), FALSE, pkg)
                except up2date.RpmError, e:
                    dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
                    
            self.skippedArea.thaw()

            self.gotSkippedPackages = TRUE

    def onSkippedPageNext(self, page, dummy):
        pass
    
    def onPackagePagePrepare(self, page, dummy):
        self.packageArea.freeze()
        self.packageArea.clear()
        self.totalSelectedSize = long(0)

        if len(self.skipPkgList) > 0:
            for index in range(self.skippedArea.n_rows):
                rowData = self.skippedArea.get_row_data(index)
                if rowData[0] == 1:
                    pkg = rowData[1]
                    self.packageArea.append_row((pkg[0],
                                                 pkg[1],
                                                 pkg[2],
                                                 "%d kb" % (pkg[4] / 1024)), TRUE, pkg)
                    try:
                        self.totalSelectedSize = self.totalSelectedSize + pkg[4]
                    except up2date.RpmError, e:
                        self.fatalError(_("There was a fatal RPM error. The message was:\n") + e.errmsg)
        elif len(self.pkgList) == 0:
            dlg = gnome.ui.GnomeOkDialog(_("Your system is fully updated.  No new packages are needed."), self.mainWin)
            dlg.run_and_close()
            sys.exit(0)
            
        for pkg in self.pkgList:
            try:
                self.packageArea.append_row((pkg[0], pkg[1], pkg[2],
                                             "%d kb" % (pkg[4] / 1024)), FALSE, pkg)
            except up2date.RpmError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)

        self.xml.get_widget("sizeLabel").set_text("%d kb" % (self.totalSelectedSize / 1024))
        self.packageArea.thaw()
        

    def onPackagePageNext(self, page, dummy):
        def cb(foo):
            return foo
        
        if self.totalSelectedSize > up2date.freeDiskSpace():
            dlg = gnome.ui.GnomeErrorDialog(_("The total size of selected packages is %d kb, but\nYou only have %d kb of free space.\nPlease unselect some packages and continue.") % (self.totalSelectedSize / 1024, up2date.freeDiskSpace() / 1024),
                                            self.mainWin)
            return TRUE

        if self.totalSelectedSize == 0:
            dlg = gnome.ui.GnomeErrorDialog(_("You must select at least one package.  If you do not wish\nto update any packages, press the \"Cancel\" button to exit\nUpdate Agent."), self.mainWin)
            return TRUE

        # build list of selected packages
        rowData = ()
        self.selectedPkgList = []
        newPackages = []
        for index in range(self.packageArea.n_rows):
            rowData = self.packageArea.get_row_data(index)
            if rowData[0] == 1:
                self.selectedPkgList.append(rowData[1])

        pwin = progress.Progress()

        while gtk.events_pending():
            gtk.mainiteration(FALSE)
        try:
            (self.depPackages,self.conflicts) = up2date.dryRun(self.selectedPkgList,
                                                               pwin.setLabel,
                                                               pwin.setProgress,
                                                               self.__refreshCallback)
        except up2date.CommunicationError, e:
            pwin.destroy()
            self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
        except up2date.RpmError, e:
            pwin.destroy()
            dlg = self.fatalError(
                _("There was an RPM error.  The message was:\n") + e.errmsg
                )
        except up2date.DependencyError, e:
            pwin.destroy()
            dlg = gnome.ui.GnomeWarningDialog(
                _("There was a package dependency problem.  The message was:\n") + \
                e.errmsg + \
                _("\nPlease modify your package selections and try again.")
                )
            return TRUE            
        except up2date.ConflictError, e:
            pwin.destroy()
            dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
            return TRUE
            
        pwin.destroy()
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # er, handle conflicts?

        if len(self.depPackages) > 0:
            self.druid.set_page(self.xml.get_widget("dependencyPage"))
            return TRUE
        else:
            self.druid.set_page(self.xml.get_widget("retrievalPage"))
            return TRUE


    def onPackagePageBack(self, page, dummy):
        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("startPage"))
        return TRUE


    def onDependencyPagePrepare(self, page, dummy):
        area1 = self.xml.get_widget("dependencyArea1")
        area1.clear()
        for pkg in self.depPackages:
            area1.append(pkg)
	    self.totalSelectedSize = self.totalSelectedSize + pkg[4]

        if self.totalSelectedSize > up2date.freeDiskSpace():
            dlg = gnome.ui.GnomeErrorDialog(_("The total size of selected packages is %d kb, but\nYou only have %d kb of free space.\nPlease unselect some packages and continue.") % (self.totalSelectedSize / 1024, up2date.freeDiskSpace() / 1024),
                                            self.mainWin)
            self.druid.set_page(self.xml.get_widget("packagePage"))
	    return TRUE


    def setRetrievalProgress(self, amount, total, speed = 0, secs = 0):
        if total:
            i = float(amount) / total
        else:
            i = 1
            
        self.progress.update(i)

        if speed:
            trans_bps_str = "%d of %d k transferred at %d k/sec\n" % \
                            (amount/1024, total/1024, speed/1024)
            totalTime = total / speed
            trans_time_str = "Package transfer time: %02d:%02d:%02d" % \
                             (totalTime / 3600, (totalTime % 3600) / 60,
                              totalTime % 60)
            trans_rem_str = " (%02d:%02d:%02d remaining)" % \
                            (secs / 3600, (secs % 3600) / 60,
                             (secs % 60))
            tstr = trans_bps_str + trans_time_str + trans_rem_str
        else:
            tstr = _("Package already downloaded")

        self.xml.get_widget("timeLabel").set_text(tstr)
        
        while (gtk.events_pending()):
            gtk.mainiteration(FALSE)

    def startRetrievalTimer(self):
        pass

    def labelcb(self, name, shortName):
        label = self.xml.get_widget("retrievePackageNameLabel")
        label.set_text(name)
            

    def doRetrieval(self):

        def cb(arg):
            return arg

        gtk.threads_enter()
        self.progress = self.xml.get_widget("retrievePackageProgress")
        totalProgress = self.xml.get_widget("retrieveTotalProgress")
        totalProgress.update(0)
        

        self.setBusyCursor()
        
        pkgCount = len(self.selectedPkgList)
        totalRetrievedSize = 0
        for index in range(pkgCount):
            self.startRetrievalTimer()
            hdr = up2date.getHeader(self.selectedPkgList[index],lite=1)
            self.xml.get_widget("retrieveInfoLabel").set_text("%s\n%s" %
                                                              (hdr['summary'],
                                                              hdr['description']))
            try:
                up2date.getPackage(self.selectedPkgList[index],
                                   self.labelcb,
                                   self.setRetrievalProgress)
            except up2date.CommunicationError, e:
                self.fatalError(_("Error while retrieving package %s.\nThe message was:\n%s") % (up2date.pkgToString(self.selectedPkgList[index]), e.errmsg))
                
            
            # do signature checking only if GPG installation checks out,
            # and GPG checking is enabled.
            # we already warned them about any errors when we started up.
            if up2date.cfg.readEntry("useGPG") and not up2date.checkGPGInstallation():
                try:
                    res = up2date.hasBadSignature(self.selectedPkgList[index])
                except up2date.RpmError, e:
                    self.fatalError(_("A fatal RPM error has occurred.  The message was:\n%s") % e.errmsg)
                if res == 1:
                    dlg = gnome.ui.GnomeQuestionDialog(_("The package %s is not signed with a GPG signature.  Continue?") % up2date.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)
                if res == 2:
                    dlg = gnome.ui.GnomeQuestionDialog(_("The package %s does not have a valid GPG signature.\nIt has been tampered with or corrupted.  Continue?") % up2date.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)

            totalRetrievedSize = totalRetrievedSize + \
                                 self.selectedPkgList[index][4]
            
            i = float(totalRetrievedSize) / self.totalSelectedSize
            totalProgress.update(i)
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

        self.xml.get_widget("packagesFinishedLabel").set_text(_("All finished.  Click \"Next\" to continue."))
        self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0


    def onRetrievalPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doRetrieval)


    def onRetrievalPageNext(self, page, dummy):
        if up2date.retrieveOnly():
            self.druid.set_page(self.xml.get_widget("finishPage"))
        else:
            self.druid.set_page(self.xml.get_widget("installPage"))
        return TRUE

    def onRetrievalPageBack(self, page, dummy):
        self.xml.get_widget("packagesFinishedLabel").set_text("...")
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE
    

    def rpmCallback(self, what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                self.fd = os.open(fileName, os.O_RDONLY)
            except:
                raise up2date.RpmError(_("Error opening %s") % fileName)
            
            return self.fd
        elif what == rpm.RPMCALLBACK_INST_START:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            self.pkgIndex = self.pkgIndex + 1
            self.pkgLabel.set_text(fileName)
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            os.close(self.fd)
            # we used to do a up2date.remoteAddPackage() here
            
            i = (self.pkgIndex * 1.0) / self.pkgCount
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.totalProgress.update(i)

        elif what == rpm.RPMCALLBACK_INST_PROGRESS:
            i = (amount + 1.0) / total
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.progress.update(i)
        elif what == rpm.RPMCALLBACK_UNINST_STOP:
            # we used to do a up2date.remoteDelPackage() here
            pass
        
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def doInstallation(self):
        gtk.threads_enter()
        self.progress = self.xml.get_widget("installPackageProgress")
        self.totalProgress = self.xml.get_widget("installTotalProgress")
        self.totalProgress.update(0)
        self.pkgLabel = self.xml.get_widget("installPackageNameLabel")
        self.pkgIndex = 0
        self.pkgCount = len(self.selectedPkgList)

        self.setBusyCursor()
        try:
            kernelsToInstall = up2date.installPackages(self.selectedPkgList, self.rpmCallback)
        except up2date.RpmError, e:
            self.setArrowCursor()
            self.fatalError(_("There was a fatal RPM error.  The message was:\n") + e.errmsg)

        
        if len(kernelsToInstall) and lilocfg:
            try:
                up2date.installBootLoader(kernelsToInstall)
            except lilocfg.LiloConfError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
            except lilocfg.LiloInstallError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
                
        if not up2date.cfg.readEntry("keepAfterInstall"):
            for pkg in self.selectedPkgList:
                try:
                    up2date.removePackage(pkg)
                except up2date.FileError, e:
                    dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)

                while gtk.events_pending():
                    gtk.mainiteration(FALSE)

        self.xml.get_widget("installingFinishedLabel").set_text(_("All finished.  Click \"Next\" to continue."))
        self.druid.set_buttons_sensitive(FALSE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0
    

    def onInstallPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doInstallation)


    def onInstallPageBack(self, page, dummy):
        self.xml.get_widget("installingFinishedLabel").set_text("...")
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE


    def onFinishPagePrepare(self, page, dummy):
        druid = self.xml.get_widget("druid")
        druid.set_buttons_sensitive(FALSE, TRUE, FALSE)
        druid.set_show_finish(TRUE)
        
        text = _("The Red Hat Update Agent has finished ")
        if up2date.retrieveOnly():
            text = text + _("retrieving\n the following packages successfully:\n\n")
        else:
            text = text + _("installing\n the following packages successfully:\n\n")

        for pkg in self.selectedPkgList:
            text = text + up2date.pkgToString(pkg) + "\n"

        if up2date.retrieveOnly():
            text = text + _("""
Because of your settings, these packages were only downloaded,
not installed.  If you wish to change your settings, please run the
"up2date-config" program.""")

        self.xml.get_widget("finishArea").insert_defaults(text)
        return FALSE


    def onFinishPageFinish(self, page, dummy):
        gtk.mainquit()
    

def showAbout():
    dlg = gnome.ui.GnomeAbout(_("About Red Hat Update Agent"),
                              up2date.version(),
                              "Copyright (c) 1999-2001 Red Hat, Inc.",
                              ["Preston Brown <pbrown@redhat.com>",
                               "Adrian Likins <alikins@redhat.com>"],
                              _("This software is distributed under the GPL.  Please Report bugs to Red Hat's Bug Tracking System: http://bugzilla.redhat.com"))
    dlg.run_and_close()

def rootWarning():
    dlg = gnome.ui.GnomeErrorDialog(_("You must run the Update Agent as root."))
    dlg.run_and_close()

def errorWindow(msg):
    dlg = gnome.ui.GnomeErrorDialog(msg)
    dlg.run_and_close()

def main():
    def cb(foo):
        return foo
        
    test = 0

    if len(saveargs) > 0:
        if saveargs[0] == "-t" or saveargs[0] == "--test":
            test = 1
        
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    if os.geteuid() != 0 and not test:
        rootWarning()
        sys.exit(1)

    gui = Gui()
    gtk.mainloop()

    
if __name__ == "__main__":
    try:
        main()
    except xmlrpclib.ResponseError, e:
        print e
    except IOError, e:
        print _("There was some sort of I/O error: %s") % e.errmsg
